#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
@auteur: David ALBERTO      (www.astrolabe-science.fr)
Ce script utilise le module mendeleev (données sur les éléments chimiques)
Dans ce module, un élément est appelé par element(N) pour récupérer
les données suivantes :
    - symbole
    - period : numéro de ligne
    - atomic_weight : la masse atomique (en u), ici assimilée à la masse molaire atomique (en g/mol)
"""
# Appel des modules
import matplotlib.pyplot as plt
import pandas as pd
import numpy as np# gestion des données numériques absentes (NaN)
from mendeleev import element
# ---------------------------------


 # Paramètres personnalisables-----
blocs=False # True : cases colorées selon les blocs. False : incolores
masses_molaires = True # affichage ou pas
nombre_masse = True # affichage ou pas
legende = True

# sélection des numéros atomiques à afficher :
Zmax = 118
#  exemple sans le bloc f :
# liste_Z = np.concatenate((
#     np.arange(1,56+1),
#     np.arange(71,86)
#     ))
liste_Z = np.arange(1,Zmax+1) # pour tous les éléments

plt.rcParams["font.family"] = "Roboto Condensed" # ou autre police installée

"""
Les paramètres suivants sont liés entre eux, pour le rendu final.
Ils doivent être ajustés manuellement :
    - dimensions de la figure
    - valeurs des paramètres d'échelle a et b
    - taille des caractères
    - les différents offsets en x et en y
"""
#facteurs d'échelle :
a=0.22 #en abscisses
b=0.28 #en ordonnées. NB : les ordonnées sont négatives

# Réglages d'écriture du symbole des éléments :
taille_symbole=16
c_symbole='k'#couleur symbole

# Réglages d'écriture du numéro atomique :
x_offset_Z=-0.14
y_offset_Z=-0.22
taille_Z=8
c_Z='k'#couleur numéro atomique

# Réglages d'écriture des terres rares :
y_offset_La=2.5#décalage vertical lanthanides
y_offset_Ac=y_offset_La#décalage vertical actinides

# Réglages d'écriture du nom des éléments :
x_offset_nom=0.0
y_offset_nom=-0.4
c_nom='darkblue'
taille_nom=7

# Réglages d'écriture de la masse molaire des éléments :
x_offset_masse=0.45
y_offset_masse=0.33
c_masse='darkmagenta'
taille_masse=8

# Couleurs des blocs :
c_bloc_s='thistle'
c_bloc_p='violet'
c_bloc_d='lightcyan'
c_bloc_f='lightsteelblue'

# Nombre de masse A :
c_A='orangered'
x_offset_A=x_offset_Z
y_offset_A=0.17
taille_A=8



#---------------------------------------------------------
# Deux options pour le format des valeurs de masse molaire :

def chiffSignif(n):#renvoie un nombre n avec 3 chiffres significatifs
    if n<10:
        valeur='{:.2f}'.format(n)
    elif n<100:
        valeur= '{:.1f}'.format(n)
    else:
        valeur= '{:.0f}'.format(n)
    return valeur.replace('.',',')#virgule à la place du point

def decimale(n):
    """
    renvoie un nombre avec une décimale, sous forme de chaîne
    de caractère, avec virgule décimale.
    """
    return str(round(n,1)).replace('.', ',')


def existe(fichier):
    """
    Renvoie True si le fichier existe dans le répertoire du script, False sinon.
    """
    try:
        f = open(fichier,'r')
        f.close()
        return True
    except:
        return False

def colonnes(Z):
    """
    renvoie la colonne d'un élément.
    """
    if Z <= 18:
        col = element(Z).group_id
    elif Z >= 57 and Z <= 70:#lanthanides
        col = Z - 54
    elif Z >= 89 and Z <= 102:# actinides
        col = Z - 86
    elif Z >= 71 and Z <= 86:
        col = Z - 68
    elif Z >= 103:
        col = Z - 100
    elif Z % 18 == 0:
        col = 18
    elif Z <= 86:
        col = Z % 18
    elif Z >= 87:
        col = Z - 86
    return col

def lignes(Z):
    """
    renvoie la ligne d'un élément.
    """
    if Z <= 18:
        lig = element(Z).period
    elif Z >= 57 and Z <= 70:#lanthanides
        lig = element(Z).period + y_offset_La
    elif Z >= 89 and Z <= 102:# actinides
        lig = element(Z).period + y_offset_La
    elif Z >= 71 and Z <= 86:
        lig = element(Z).period
    elif Z >= 103:
        lig = element(Z).period
    elif Z % 18 == 0:
        lig = Z // 18 + 2
    elif Z <= 86:
        lig = Z // 18 + 3
    elif Z >= 87:
        lig = element(Z).period
    return lig

def couleur_case(Z):
    """
    Renvoie la couleur de la case.
    """
    elem = element(int(Z))
    if blocs==True:
        if elem.block=='s':
            couleur =c_bloc_s
        elif elem.block=='p':
            couleur =c_bloc_p
        elif int(Z) in [57,89]:couleur = c_bloc_f
        elif int(Z) in [71,103]:couleur = c_bloc_d
        elif elem.block=='d':
            couleur =c_bloc_d
        else : couleur =c_bloc_f
    else: couleur ='lavender'
    return couleur
#---------------------------------------------------------
noms=['Hydrogène','Hélium','Lithium','Béryllium','Bore','Carbone','Azote','Oxygène',
      'Fluor','Néon','Sodium','Magnésium','Aluminium','Silicium','Phosphore','Soufre',
      'Chlore','Argon','Potassium','Calcium','Scandium','Titane','Vanadium','Chrome',
      'Manganèse','Fer','Cobalt','Nickel','Cuivre','Zinc','Gallium','Germanium',
      'Arsenic','Sélénium','Brome','Krypton','Rubidium','Strontium','Yttrium',
      'Zirconium','Niobium','Molybdène','Technétium','Ruthénium','Rhodium',
      'Palladium','Argent','Cadmium','Indium',
'Étain','Antimoine','Tellure','Iode','Xénon','Césium','Baryum','Lanthane','Cérium',
'Praséodyme','Néodyme','Prométhium','Samarium','Europium','Gadolinium','Terbium',
'Dysprosium','Holmium','Erbium','Thulium','Ytterbium','Lutetium','Hafnium','Tantale',
'Tungstène','Rhénium','Osmium','Iridium','Platine','Or','Mercure','Thallium','Plomb',
'Bismuth','Polonium','Astate','Radon','Francium','Radium','Actinium','Thorium','Protactinium',
'Uranium','Neptunium','Plutonium','Americium','Curium','Berkelium','Californium','Einsteinium',
'Fermium','Mendelevium','Nobelium','Lawrencium','Rutherfordium','Dubnium','Seaborgium','Bohrium',
'Hassium','Meitnerium','Darmstadtium','Roentgenium','Copernicium','Nihonium','Flerovium',
'Moscovium','Livermorium','Tennesse','Oganesson',
]#le module mendeleev n'a que les noms des éléments en anglais
# ---------------------------------


"""
La ligne est la valeur de la période de l'élément.
La colonne est déterminée différemment, selon la période, dans une fonction 'colonnes'.
Cette opération peut être faite une seule fois, à la 
1e exécution : les résultats sont stockés dans un fichier
csv pour être réutilisés lors des exécutions ultérieures,
pour gagner du temps.
"""

if existe('tableau.csv'):
    tableau = pd.read_csv("tableau.csv", sep='\t')
else:# creation des listes de lignes et colonnes, ainsi que du fichier csv les contenant.
    ligne = []
    colonne = []
    symbole = []
    for Z in liste_Z:
        Z = int(Z)
        col = colonnes(Z)
        lig = lignes(Z)
        symb = element(Z).symbol
        colonne.append(col)
        ligne.append(lig)
        symbole.append(symb)

    #enregistrement pour usage ultérieur, afin de gagner du temps
    tableau = pd.DataFrame(
                    {'Z' : liste_Z,
                     'nom' : noms[0:len(liste_Z)],
                     'col' : colonne,
                     'lig' : ligne,
                     'symbole' : symbole
                        }
        )
    tableau.to_csv('tableau.csv',index=False, sep='\t')

# ---------------------------------
# paramètres du graphique
fig=plt.figure(figsize=(29.7/2.54,21/2.54),tight_layout=True,facecolor='cornflowerblue')

# plt.rcParams["font.family"] = "Liberation Serif"# ou autre police installée (police Serif conseillée)

# plt.rcParams.update({"text.usetex": True}) # utiliser la police LaTeX si installée (optionnel)
ax=plt.subplot(111)

plt.title("Tableau périodique des éléments chimiques",fontsize=20,y=0.90)
# plt.title("Tableau périodique des éléments chimiques",fontsize=20,y=0.90)
plt.xlim(0,19*a)
plt.ylim(-10.5*b,0)
plt.xticks([])
plt.yticks([])
ax.set_aspect('equal')

# ---------------------------------

# Remplissage du tableau :
for Z in liste_Z:
    x = tableau.loc[tableau['Z']==Z, 'col'] * a
    y = - tableau.loc[tableau['Z']==Z,'lig'] * b
    nom = tableau.loc[tableau['Z']==Z,'nom'].values[0]
    symbole = tableau.loc[tableau['Z']==Z,'symbole'].values[0]
    plt.text(x,y,symbole,fontsize=taille_symbole,va='center',ha='center', c = c_symbole) # symbole

    elem = element(int(Z))
    coul_case = couleur_case(Z)
    rectangle=plt.Rectangle((x-0.5*a, y-0.5*b),a,b,facecolor=coul_case,edgecolor='white',linewidth=2)#case
    ax.add_patch(rectangle)
    if int(Z) in [91,101,104,110,111,116]:#éléments aux noms longs
        tailleDuNom = taille_nom - 1
    else:tailleDuNom = taille_nom
    plt.text(x+x_offset_Z*a,y+y_offset_Z*b,Z,c=c_Z,fontsize=taille_Z,ha='right')#numéro atomique
    plt.text(x+x_offset_nom*a,y+y_offset_nom*b,'%s'%(nom),c=c_nom,fontsize=taille_nom,ha='center')#nom
    # autre option, sans le code LaTeX :
    # plt.text(x+x_offset_nom*a,y+y_offset_nom*b,nom,c=c_nom,fontsize=tailleDuNom,ha='center')#nom
    if masses_molaires==True:
        plt.text(x+x_offset_masse*a,y+y_offset_masse*b,
        decimale(elem.atomic_weight),c=c_masse,fontsize=taille_masse,ha='right')#masse molaire
    if nombre_masse==True:
        plt.text(x+x_offset_A*a,y+y_offset_A*b,
        elem.mass_number,c=c_A,fontsize=taille_A,ha='right')#nombre de masse
    tailleDuNom=taille_nom

# Légende :
if legende:
    x=7*a   # abscisse de la légende
    y=-b*2  # ordonnée de la légende
    Z=12    # choix de l'élément chimique
    nom = tableau.loc[tableau['Z']==Z,'nom'].values[0]
    elem=element(Z)
    plt.text(x,y,elem.symbol,fontsize=taille_symbole,va='center',ha='center')#symbole
    couleur_case='none'
    rectangle=plt.Rectangle((x-0.5*a, (y-0.5*b)),a,b,facecolor=couleur_case,edgecolor='k')#case
    ax.add_patch(rectangle)
    tailleDuNom=taille_nom
    plt.text(x+x_offset_Z*a,y+y_offset_Z*b,Z,c=c_Z,fontsize=taille_Z,ha='right')#numéro atomique
    plt.text(x+x_offset_nom*a,y+y_offset_nom*b,'%s'%(nom),c=c_nom,fontsize=tailleDuNom,ha='center')#nom
    # autre option, sans le code LaTeX :
    # plt.text(x+x_offset_nom*a,y+y_offset_nom*b,nom,c=c_nom,fontsize=taille_nom,ha='center')#nom
    if masses_molaires==True:
        plt.text(x+x_offset_masse*a,y+y_offset_masse*b,
        decimale(elem.atomic_weight),c=c_masse,fontsize=taille_masse,ha='right')#masse molaire
    if nombre_masse==True:
        plt.text(x+x_offset_A*a,y+y_offset_A*b,
        elem.mass_number,c=c_A,fontsize=taille_A,ha='right')#nombre de masse
    
    
    plt.annotate('numéro atomique Z', (x-a/2*0.8,y+y_offset_Z*b),
                 xytext=(x-1*a,y+y_offset_Z*b-0.5*b),horizontalalignment="right",
                 c=c_Z,arrowprops=dict(arrowstyle='-',color=c_Z))
    if masses_molaires==True:
        plt.annotate(r'masse molaire (g.mol$^{-1}$)', (x+a/2*0.85,y+y_offset_masse*b),
                     xytext=(x+2*a,y+y_offset_masse*b-0.5*b),horizontalalignment="center",c=c_masse,fontsize=taille_masse,arrowprops=dict(arrowstyle='-',color=c_masse))
    if nombre_masse==True:
        plt.annotate('nombre de masse A\n (du principal isotope)', (x-a/2*0.8,y+y_offset_A*b),
                     xytext=(x-1.5*a,y+y_offset_A*b),horizontalalignment="right",
                     c=c_A,arrowprops=dict(arrowstyle='-',color=c_A))

# Trait pour les terres rares :
plt.plot([2.5*a,2.5*a],[-7.5*b,-8.0*b],'--',c='gray',lw=1)

# signature :
plt.text(0.985,0.02,'D. Alberto (www.astrolabe-science.fr)',
         rotation=90,fontsize=8,transform=ax.transAxes)

# Numéros des colonnes :
for col in range(1,19):
    if col > 2 and col < 13:
        ligne = -3.4 * b
    elif col==1 or col== 18 :
        ligne = -0.4 * b
    else: ligne = -1.4 * b
    ax.text(a * col,ligne,f'{col}',color='darkgray',fontsize=10,fontweight='bold',
             ha='center')

NomFichier="TableauPeriodique"
fig.savefig(NomFichier+'.png',dpi=300)
fig.savefig(NomFichier+'.pdf')
